// global
    cuppa.global = {};
/*  dataCenter
    add and remove data in one place
    Examples:
    cuppa.setData('NEW_CLIENT', {data:{name:"Tufik", age":18}})
    cuppa.getData('NEW_CLIENT', {callback:listener});
*/
    cuppa.dataDefault = {};
    cuppa.data = {};
    cuppa.setData = function(name, opts){
        opts = cuppa.mergeObjects([{storage:'', silence:false, data:null, "default":null}, opts]);
        if(opts.store != undefined) opts.storage = opts.store;

        if(opts["default"] !== null){
            cuppa.dataDefault[name] = opts["default"];
            opts["default"] = null;
            var current = cuppa.getData(name, opts);
            if(current == null || current == undefined) current = cuppa.dataDefault[name];
            opts.data = current;
            cuppa.setData(name, opts);
            return;
        }

        cuppa.data[name] = opts.data;
        if(opts.storage === "local"){
            cuppa.localStorage(name, cuppa.jsonEncode(opts.data, false));
        }else if(opts.storage === "session"){
            cuppa.sessionStorage(name, cuppa.jsonEncode(opts.data, false));
        }else if(opts.storage === "cookie"){
            cuppa.setCookie(name, cuppa.jsonEncode(opts.data, false));
        };

        if(!opts.silence) cuppa.executeListener(name, opts.data);
    };

    cuppa.getData = function(name, opts){
        opts = cuppa.mergeObjects([{storage:'', callback:null, "default":false}, opts]);
        if(opts.store != undefined) opts.storage = opts.store;

        if(opts["default"] === true){ return cuppa.dataDefault[name]; }

        var data = cuppa.data[name];
        if(opts.storage === "local"){
            let ls = cuppa.localStorage(name);
            if(ls) data = cuppa.jsonDecode(ls, false);
        }else if(opts.storage === "session"){
            let st = cuppa.sessionStorage(name);
            if(st) data = cuppa.jsonDecode(st, false);
        }else if(opts.storage === "cookie"){
            let st = cuppa.getCookie(name);
            if(st) data = cuppa.jsonDecode(st, false);
        }

        if(data == undefined && opts["default"] !== false && opts["default"] !== true){
            data = opts["default"];
        }

        if(data != undefined && opts.callback){
            opts.callback(data) ;
        };
        if(opts.callback){
            cuppa.addListener(name, opts.callback);
        };
        return data;
    };

    cuppa.getDataSync = function(name, opts){
        opts = cuppa.mergeObjects([{"default":false}, opts]);
        var data = cuppa.data[name];
        if(opts["default"] === true){ data = cuppa.dataDefault[name]; }
        if(opts["default"] !== true && opts["default"] !== false && !data) data = opts["default"];
        return data;
    };

    cuppa.deleteData = function(name, opts){
        opts = cuppa.mergeObjects([{storage:''}, opts]);
        if(opts.store != undefined) opts.storage = opts.store;
        cuppa.data[name] = null;
        if(opts.storage === "local"){
            localStorage.removeItem(name);
        }else if(opts.storage === "session"){
            sessionStorage.removeItem(name);
        }else if(opts.storage === "cookie"){
            cuppa.deleteCookie(name);
        };
    };

// add / remove / execute global listeners
    cuppa.listeners = {};
    cuppa.addListener = function(name, callback){
        if(!cuppa.listeners[name]) cuppa.listeners[name] = [];
        cuppa.listeners[name].push(callback);
    };
    cuppa.removeListener = function(name, callback, toString){
        if(!cuppa.listeners[name]) cuppa.listeners[name] = [];
        var array = cuppa.listeners[name];
        for(var i = 0 ; i < array.length; i++ ){
            if(toString){
                if(array[i].toString() === callback.toString()){
                    array.splice(i, 1);
                };
            }else{
                if(array[i] === callback){
                    array.splice(i, 1);
                };
            }
        };
    };
    cuppa.removeListenerGroup = function(name){
        delete cuppa.listeners[name];
    };
    cuppa.executeListener = function(name, data){
        if(!cuppa.listeners[name]) cuppa.listeners[name] = [];
        var array = cuppa.listeners[name];
        for(var i = 0 ; i < array.length; i++ ){
            array[i](data);
        };
    };
/* Make Object Available to Dispach Event
    cuppa.dispatchEvents(element); // add methods to manage the dispachers.
    // dispache an event
        element.dispatchEvent({ type: "MY_EVENT", message: "My Message" });
*/
    cuppa.dispatchEvents = function(element){
        if(element && !element.dispatchEvent){ Object.assign(element, EventDispatcher.prototype); };
    }; cuppa.dispatchEvents(cuppa);
/* on / off
    EventManager, Estructure
    cuppa.eventGroups =	{ 'groupName':  Map<Element>:[{event:String, callback:Function}, {event:String, callback:Function}, ]
                                        Map<Element>:[{event:String, callback:Function}, {event:String, callback:Function}, ]
*/
    cuppa.eventGroups = [];
    // Add Event listener
        cuppa.on = function(elements, event, callback, groupName, useCapture) {
            if(!elements) return;
            if(event === "removed") event = "DOMNodeRemoved";
            elements = cuppa.element(elements);
            cuppa.off(elements, event, callback, groupName); // prevent duplicate events
            if(!groupName) groupName = "default";
            if(useCapture === undefined) useCapture = false;

            if(!cuppa.eventGroups[groupName]) cuppa.eventGroups[groupName] = new Map();
            for(var i = 0; i < elements.length; i++){
                var element = elements[i];
                if(!element) continue;
                if(element.addEventListener) element.addEventListener(event, callback, useCapture);
                var events = cuppa.eventGroups[groupName].get(element);
                if(!events) events = [];
                events.push({event:event, callback:callback});
                cuppa.eventGroups[groupName].set(element, events);
            }
        };
     // Remove a single event
        cuppa.off = function(elements, event, callback, groupName){
            elements = cuppa.element(elements);
            if(!groupName) groupName = "default";
            if(!cuppa.eventGroups[groupName]) return;
            if(!elements) return;
            if(event === "removed") event = "DOMNodeRemoved";
            for(var i = 0; i < elements.length; i++){
                var events = cuppa.eventGroups[groupName].get(elements[i]);
                if(!events) break;
                for(var j = events.length-1; j >= 0; j--){
                    if(callback){
                        if(events[j].event === event && events[j].callback === callback ){
                            elements[i].removeEventListener(events[j].event, events[j].callback);
                            events.splice(j, 1);
                            break;
                        };
                    }else{
                        if(events[j].event === event ){
                            elements[i].removeEventListener(events[j].event, events[j].callback);
                            events.splice(j, 1);
                        };
                    };
                };
                cuppa.eventGroups[groupName].set(elements[i], events);
            };
        };
    // Remove event by Group
        cuppa.offGroup = function(groupName){
            if(!groupName) groupName = "default";
            var map = cuppa.eventGroups[groupName];
            if(!map) return;
            map.forEach(function(events, element) {
                for(var i = 0; i < events.length; i++){
                    element.removeEventListener(events[i].event, events[i].callback);
                };
            });
            map["delete"](groupName);
        };
/*  Remove duplicate in Array
    array: cuppa.removeDuplicate(array)
*/
    cuppa.removeDuplicate = function(array){
        array = array.filter((thing,index) => {
            return index === array.findIndex(obj => {
                return JSON.stringify(obj) === JSON.stringify(thing);
            });
        });
        return array;
    }
/* Clone object */
    cuppa.cloneObject = function(object, useJSON){
        if(useJSON === undefined) useJSON = true;
        var object2;
        if(useJSON) object2 = JSON.parse(JSON.stringify(object));
        else object2 = Object.assign({}, object);
        return object2;
    };
// ObjectToURL
    cuppa.objectToURL = function(object){
        var str = "";
        for(var key in object) {
            str += "&";
            if(object[key]) str += key + "=" + (object[key]);
            else str += key;
        };
        return str;
    };
// object Length
    cuppa.objectLength = function(object){
        return Object.keys(object).length;
    };
// clone Array
    cuppa.cloneArray = function(array){
        return array.slice(0);
    };
    cuppa.arrayClone = function(array){ return cuppa.cloneArray(array); };
// arrayTomap
    cuppa.arrayToMap = function(array, value, opts){
        opts = cuppa.mergeObjects([{lowerCase:false}, opts]);
        var map = {};
        for(var i = 0; i < array.length; i++){
            let key = array[i][value];
            if(opts.lowerCase) key = key.toLowerCase();
            map[key] = array[i];
        }
        return map;
    };
/* Get value of string with URLFormat
    Example: &var1=mi_var1&var2=mi_var2&var3=mi_var3
            cuppa.urlValue('var2')
            return mi_var2
*/
	cuppa.urlValue = function(name, url){
        if(!url) url = cuppa.getPath();
        /* new way
            var url = new URL(url);
            var value = url.searchParams.get(name);
            return value;
        */
		var regexS = "[\\?&]"+name+"=([^&#]*)";
		var regex = new RegExp(regexS,"i");
		var tmpURL = url;
        var result = regex.exec(tmpURL);
        if(result == null) return "";
            result = result[1];
            result = cuppa.replace(result, "%20", " ");
            result = cuppa.replace(result, "\\+", " ");
		return result;
	};
/* Short Object ASC or DESC */
    cuppa.shortObject = function(object, reverse){
        var keys = Object.keys(object);
        keys.sort();
        if(reverse) keys.reverse();
        var tmpObject = {}
        for(var i = 0; i < keys.length; i++){
            tmpObject[keys[i]] = object[keys[i]];
        };
       return tmpObject;
    };
// swich img.svg to svg
cuppa.imgToSVG = cuppa.img2SVG = function(elements, callback){
    if(!elements) elements = ".svg";
    elements = cuppa.element(elements);
    for(var i = 0; i < elements.length; i++){
        cuppa.svgSwitch(elements[i], callback);
    };
};
    cuppa.svgSwitch = function(element, callback){
        var src = element.getAttribute("src"); if(!src) return;
        var classNames = element.getAttribute("class");
        var request = new XMLHttpRequest();
            request.onload = onComplete;
            request.open("GET", element.getAttribute("src"));
            request.send();
            function onComplete () {
                var parent = element.parentNode;
                var temp = document.createElement('div');
                    temp.innerHTML = this.responseText;
                var svg = temp.querySelector("svg");
                    if(!svg){ log("[cuppa.svgSwitch] error to load SVG on: ",element); return; }
                    svg.setAttribute("class", classNames);
                    if(classNames && parent) parent.replaceChild(svg, element);
                if(callback) callback(svg);
            };
    };
/* menu
    type:1, menuAnchor:".cu-menu-anchor"
*/
    cuppa.menu = function(opts){
        this.opts = {type:1, menu:'.cu-menu', gap:0, anchor:".cu-menu-anchor"};
        this.opts = cuppa.mergeObjects([this.opts, opts]);
        this.element = cuppa.element(this.opts.menu, {query:true})[0];
        this.anchor = cuppa.element(this.opts.anchor, {query:true})[0];
        if(!this.element) return;
        // show fixed menu if anchor.y <= 0;
        // show normal menu if scroll.y = 0;
        this.type1 = function(){
            var scrollPos = cuppa.scrollPosition();
            if(!this.anchor) return;
            var anchorPos = cuppa.dim(this.anchor);
            if( this.anchor.toString() == "[object HTMLBodyElement]") anchorPos.y3 = -99999;
            if(anchorPos.y3 + this.opts.gap <= 0){
                if(this.element.fixed) return;
                this.element.fixed = true;
                var dim = cuppa.dim(this.element);
                cuppa.addClass(this.element, "cu-menu-fixed-top");
                window.TweenMax.killTweensOf(this.element);
                var tl = new window.TimelineMax();
                    tl.fromTo(this.element, 0.4, {top:-dim.height},{top:0});
            }else if(!scrollPos.y){
                if(!this.element.fixed) return;
                this.element.fixed = false;
                cuppa.removeClass(this.element, "cu-menu-fixed-top");
            }
        }.bind(this);

        this.onScroll = function(e){
            if(this.opts.type == 1) this.type1();
        }.bind(this); cuppa.scroll(this.onScroll); this.onScroll();
    };
/* footer fixed */
    cuppa.footerFixed = function(element){
        if(!element) element = ".footer-fixed";
        element = cuppa.element(element, {query:true})[0];
        if(!element) return;
        function ajust(e){
            var dim = cuppa.dim(element);
            cuppa.css(document.body, {"margin-bottom":dim.height+"px"});
        }; cuppa.on(window, "resize", ajust); cuppa.on(window, "load", ajust); ajust();
    }
/* autoDimention
    Example:
    <div data-cu-height="100%">
    <script> new cuppa.autoDimention(); </script>
*/
    cuppa.autoDimention = function(elements){
        this.elements = elements;
        // get elements in dom
            if(this.elements == undefined){
                this.elements = [];
                this.elements = this.elements.concat(cuppa.element("[data-cu-height]"));
                this.elements = this.elements.concat(cuppa.element("[data-cu-width]"));
            }else if(!Array.isArray(elements)){
                this.elements = [elements]
            };
        // resize
            this.resize = function(){
                for(var i = 0; i < this.elements.length; i++){
                    var element = this.elements[i];
                    var dim = cuppa.dim(element);
                    // height
                        var attr = element.getAttribute("data-cu-height");
                        if(attr){
                            if(cuppa.search("%", attr)){
                                var value = dim.width*(parseFloat(attr)/100);
                                cuppa.css(element,{height:value+"px"});
                            };
                            if(cuppa.search("vh", attr)){
                                var value = cuppa.height()*(parseFloat(attr)/100);
                                cuppa.css(element,{height:value+"px"});
                            };
                            if(cuppa.search("vw", attr)){
                                var value = cuppa.width()*(parseFloat(attr)/100);
                                cuppa.css(element,{height:value+"px"});
                            };
                        };
                    // width
                        var attr = element.getAttribute("data-cu-width");
                        if(attr){
                            if(cuppa.search("vw", attr)){
                                var percent = parseFloat(attr)/100;
                                var width = cuppa.width()*percent;
                                cuppa.css(element,{"width":width+"px"});
                                cuppa.css(element,{"margin-left":"0px"});
                                var dim = cuppa.dim(element);
                                var x = -dim.x + ( cuppa.width() - width )*0.5;
                                cuppa.css(element, {"margin-left":x+"px"});
                            };
                            if(cuppa.search("%", attr)){
                                var percent = parseFloat(attr)/100;
                                var parent = cuppa.parent(element);
                                var dim = cuppa.dim(parent);
                                cuppa.css(element,{width:dim.width*percent+"px"});
                            };
                        };
                };
            }.bind(this); window.addEventListener("resize", this.resize); window.addEventListener("load", this.resize); this.resize();
    }; cuppa.autoDim = function(elements){ new cuppa.autoDimention(elements); };
/* is Mobile */
    cuppa.isMobile = function() {
        var check = false;
        (function(a){if(/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i.test(a)||/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0,4))) check = true;})(navigator.userAgent||navigator.vendor||window.opera);
        return check;
    };
// Touch support
    cuppa.isTouch = cuppa.touchSupport = function() {
        if ('ontouchstart' in document.documentElement) { return true; }else{ return false; }
    };
/* flash support */
    cuppa.flashSupport = function(){
        var hasFlash = false;
        try {
            var fo = new window.ActiveXObject('ShockwaveFlash.ShockwaveFlash');
            if (fo) { hasFlash = true; }
        } catch (e) {
            if (navigator.mimeTypes && navigator.mimeTypes['application/x-shockwave-flash'] != undefined && navigator.mimeTypes['application/x-shockwave-flash'].enabledPlugin) {
                hasFlash = true;
            };
        };
        return hasFlash;
    };
/* join an array / object in a string */
    cuppa.join = function(item, glue){
        if(glue == undefined) glue = ",";
        var string = "";
        var arr = Object.keys(item).map(function (key) { return item[key]; });
            arr.forEach(function(item, index){
                if(index) string += glue;
                string += item;
            });
        return string;
    };
/* parseString */
    cuppa.parseString = function(value){
        if(!value) return "";
        else return String(value);
    };
/* parseInt */
    cuppa.parseInt = function(string){
        if(isNaN(string) || !string) return 0;
        else return parseInt(string);
    };
/* parseFloat */
    cuppa.toNumber = function(string){
        if(isNaN(string) || !string) return 0;
        else return parseFloat(string);
    };
    cuppa.parseFloat = function(string){ return cuppa.toNumber(string); }
/* isNumber */
    cuppa.isNaN = function(value){
        return isNaN(parseFloat(value));
    };
/* scroll */
    cuppa.scroll = function(callback, element){
        if(!element || element == "body") element = window;
        if(document && element == document.body) element = window;
        element.addEventListener('scroll', callback);
    };
    // get Position
    cuppa.scrollPosition = function(element){
        var pos = {x:0, y:0}
        if(!element || element == "body" || cuppa.elementType(element) == "body" || element == window){
            pos.x = window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft || 0;
            pos.y = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0
        }else{
            pos.x = element.scrollLeft;
            pos.y = element.scrollTop;
        };
        return pos;
    };
    // get Percent
    cuppa.scrollPercent = function(element){
        if(element == undefined) element = document.body;
        var element = cuppa.element(element)[0];
        var percent = {x:0, y:0};
        var position = cuppa.scrollPosition(element);
            // x
            percent.x = position.x / (element.clientWidth - window.innerWidth);
            if(percent.x < 0) percent.x = 0; if(percent.x > 1) percent.x = 1;
            // y
            percent.y = position.y / (element.clientHeight - window.innerHeight);
            if(percent.y < 0) percent.y = 0; if(percent.y > 1) percent.y = 1;
        return percent;
    }
    /* move
        opts = {
            x:0, // -1 or null or 'dont set' = no scrolling
            y:0, // -1 or null or 'dont set' = no scrolling
            duration:0.3,
            delay:0,
            ease:window.Cubic.easeInOut,
            percent:false, // if true x & y =  0 to 1
            progressCallback:function,
            completeCallback:function,
        }
    */
    cuppa.scrollMove = function(target, opts){
        if(!target) target = document.body;
        else target = cuppa.element(target, {returnType:"first"});
        if(!target) return;
        var scrollX = true, scrollY = true;
        if(opts == undefined) opts = {x:null, y:0};
        if(opts.x == undefined || opts.x == null || opts.x == -1){ scrollX = false; opts.x = 0; }
        if(opts.y == undefined || opts.y == null || opts.y == -1){ scrollY = false; opts.y = 0; }
        // opts
            opts.target = target;
            opts = opts || {};
            opts.duration = (opts.duration == undefined) ? 0.3 : opts.duration;
            opts.ease = opts.ease || window.Linear.easeNone;
            opts.percent = opts.percent || false;
            opts.delay = opts.delay || 0;
        // percent
            if(opts.percent){
                var dim = cuppa.dim(target);
                if(cuppa.elementType(target) == "body"){
                    opts.x = (dim.width-window.innerWidth)*opts.x;
                    opts.y = (dim.height-window.innerHeight)*opts.y;
                }else{
                    opts.x  = 0; opts.y = 0;
                };
            };
        // scroll
            if(cuppa.elementType(target) == "body") target = "html, body";
            if(scrollX && scrollY){
                window.TweenMax.to(target, opts.duration, { delay:opts.delay, scrollLeft:opts.x, scrollTop:opts.y, ease: opts.ease, onUpdate:onProgress, onUpdateParams:[opts], onComplete:onComplete, onCompleteParams:[opts] } );
            }else if(scrollX){
                window.TweenMax.to(target,  opts.duration, { delay:opts.delay, scrollLeft:opts.x, ease: opts.ease, onUpdate:onProgress, onUpdateParams:[opts], onComplete:onComplete, onCompleteParams:[opts] } );
            }else if(scrollY){
                window.TweenMax.to(target,  opts.duration, { delay:opts.delay, scrollTop:opts.y, ease: opts.ease, onUpdate:onProgress, onUpdateParams:[opts], onComplete:onComplete, onCompleteParams:[opts] } );
            };
        function onProgress(opts){ if(opts.progressCallback) opts.progressCallback(opts); };
        function onComplete(opts){ if(opts.completeCallback) opts.completeCallback(opts); };
    };
    cuppa.moveScroll = function(target, opts){ cuppa.scrollMove(target, opts); };
/* Visible on screen
    element: html node
    gap: 0 (number)
    mode: visible, above, below
*/
   cuppa.visible = function(element, gap, mode) {
        gap = gap || 0;
        mode = mode || 'visible';
        let rect = element.getBoundingClientRect();
        let viewHeight = window.innerHeight;
        let above = rect.bottom - gap < 0;
        let below = rect.top - viewHeight + gap >= 0;
        return mode === 'above' ? above : (mode === 'below' ? below : !above && !below);
    };
/* visibleAuto
        new cuppa.visibleAuto({scroll:this.sections, callback:this.visibleElement});
    */
   cuppa.visibleAuto = function(opts){
    this.opts = cuppa.mergeObjects([{elements:null, callback:null, scroll:window, gap:0, mode:null},opts]);
    this.opts.elements = this.opts.elements || ".cu-visible";
    this.opts.elements = cuppa.element(this.opts.elements);
    this.validate = function(){
        for(let i = 0; i <= this.opts.elements.length; i++){
            let element = this.opts.elements[i];
            if(element){
                if(element.visible == undefined) element.visible = false;
                let visible = cuppa.visible(element, this.opts.gap, this.opts.mode);
                if(visible == true && !element.visible){
                    element.visible = true;
                    if(this.opts.callback) this.opts.callback(element, element.visible);
                }else if(visible == false && element.visible){
                    element.visible = false;
                    if(this.opts.callback) this.opts.callback(element, element.visible);
                }
            }
        }
    };
    cuppa.on(this.opts.scroll, "scroll", this.validate.bind(this)); this.validate();
}

/* mouseWheelSlider
    new cuppa.mouseWheelSlider({elements:".hybridWrap .section",target:this.sections});
*/
    cuppa.mouseWheelSlider = function(opts){
        this.opts = cuppa.mergeObjects([{elements:null, target:window, disabled:false, invert:false, index:0, duration:0.6, ease:Power3.easeInOut, timeoutTime:1000, callback:null},opts]);
        this.opts.elements = this.opts.elements || ".cu-visible";
        this.opts.elements = cuppa.element(this.opts.elements);
        this.animating = false;
        this.onWheel = function(e){
            if(this.opts.disabled) return;
            e.preventDefault();
            let delta = e.deltaY;
            if(this.opts.invert) delta = delta*-1;
            this.change(delta);
        }

        this.change = function(delta){
            if(this.animating) return;
            this.animating = true;
            if(delta > 0){
                this.opts.index++;
            }else if(delta < 0){
                this.opts.index--;
            }
            if(this.opts.index > this.opts.elements.length-1 || this.opts.index < 0){
                this.animating = false;
                if(this.opts.index > this.opts.elements.length-1) this.opts.index = this.opts.elements.length-1;
                else if(this.opts.index < 0) this.opts.index = 0;
                return;
            }

            let element = this.opts.elements[this.opts.index];
            let dim = cuppa.dim(element);
            cuppa.scrollMove(this.opts.target, {y:dim.y2, duration:this.opts.duration, ease:this.opts.ease, completeCallback:function(){
                setTimeout(function(){
                    this.animating = false;
                    if(this.opts.callback) this.opts.callback(this.opts.index);
                }.bind(this), this.opts.timeoutTime);
            }.bind(this)});
        }
        cuppa.off(this.opts.target, "wheel", this.onWheel);
        cuppa.on(this.opts.target, "wheel", this.onWheel.bind(this));
        return this;
    };

// tooptip
    cuppa.tooltip = function(elements, opts){
        this.opts = cuppa.mergeObjects([{class:"a-r-t", style:""}, opts]);
        this.elements = elements || ".cu-tooltip";
        this.elements = cuppa.element(this.elements);

        // mouseenter
        this.mouseenter = function(e){
            let element = e.currentTarget;
            let tooltip = cuppa.newElement(`<div class='tooltip-message'></div>`);
                tooltip.innerHTML = cuppa.attr(element, "tooltip");
                let classes = cuppa.attr(element,"tooltip-class") || this.opts.class;
                    cuppa.addClass(tooltip, classes);
                let style = cuppa.attr(element,"tooltip-style") || this.opts.style;
                    cuppa.attr(tooltip, "style", style);
                element.tooltipElement = tooltip;
            cuppa.append(tooltip, element);
        }.bind(this);

        // mouseleave
        this.mouseleave = function(e){
            let element = e.currentTarget;
            cuppa.remove(element.tooltipElement);
        }.bind(this);

        for(var i = 0; i < this.elements.length; i++){
            var element = this.elements[i];
            if(element.cuTooltip) continue;
            element.cuTooltip = this;
            cuppa.on(element, "mouseenter", this.mouseenter);
            cuppa.on(element, "mouseleave", this.mouseleave);
        };
    };
/* tween
    class='cu-tween'
    attributes
        data-a-init='y:200, alpha:0'
        data-a-to='y:-100, alpha:1, duration:0.8, ease:window.Cubic.easeOut,'
        data-a-to-2='y:0, ease:window.Cubic.easeOut'
        data-a-gap="100"
        data-a-start="load"
        data-a-init-child='y:200, alpha:0'
        data-a-stagger-child='y:0, alpha:1, ease:Cubic.easeOut, duration:0.4, stagger:0.1, reverse:1'
        // animate number
            data-a-to='numberTo:200, duration:2, ease:window.Cubic.easeOut'
            data-a-to-2='numberFrom:200, numberTo:400, duration:2, ease:window.Cubic.easeOut'
*/
    cuppa.tween = function(elements, opts){
        this.opts = cuppa.mergeObjects([{loadExpire:0, scrollElement:null}, opts]);
        this.windowLoaded = false;
        // init
        this.elements = elements || ".cu-tweenmax, .cu-tween";
        this.elements = cuppa.element(this.elements);
        for(var i = 0; i < this.elements.length; i++){
            var element = this.elements[i];
            var paramsString = element.getAttribute("data-a-init");
            if(paramsString){
                var tmp = paramsString.split(",");
                var params = {};
                    tmp.map(function(element){
                        var tmp2 = element.split(":");
                        var key = cuppa.trim(tmp2[0]);
                        var value = cuppa.trim(tmp2[1]);
                        if(isNaN(value)) params[key] = value;
                        else if(key && value != undefined) params[key] = cuppa.parseFloat(value);
                    });
                window.TweenLite.set(element, params);
            };
            // if data-a-init-child set, get childrens
            var paramsStringChild = element.getAttribute("data-a-init-child");
            if(paramsStringChild){
                var childrens = element.children;
                var tmp = paramsStringChild.split(",");
                var params = {};
                    tmp.map(function(element){
                        var tmp2 = element.split(":");
                        var key = cuppa.trim(tmp2[0]);
                        var value = cuppa.trim(tmp2[1]);
                        if(isNaN(value)) params[key] = value;
                        else if(key && value != undefined) params[key] = cuppa.parseFloat(value);
                    });
                window.TweenLite.set(childrens, params);
            };
        };
        // scroll
        this.scroll = function(){
            for(var i = 0; i < this.elements.length; i++){
                var element = this.elements[i];
                var gap = cuppa.parseFloat(element.getAttribute("data-a-gap")) || 0;
                var visible = cuppa.visible(element, gap);
                var start = element.getAttribute("data-a-start");
                if(start == "load" && !this.windowLoaded) return;
                if(visible && !element.animated){
                    element.animated = true;
                    var timeLineOptions = this.getAttrData(element, "data-a-opts");
                    element.timeline = new window.TimelineMax(timeLineOptions);
                    element.timelineNumber = new window.TimelineMax(timeLineOptions);
                    var params = this.getParams(element);
                    for(var j = 0; j < params.length; j++){
                        element.timeline.to(element, params[j].duration, params[j].params);
                        // animate numbers
                            if(params[j].numberTo){
                                var n1 = { value: params[j].numberFrom || parseInt(element.innerHTML) };
                                element.timelineNumber.to(n1, params[j].duration, {value: params[j].numberTo, delay:params[j].params.delay, ease:params[j].params.ease, onUpdateParams:[n1, element],
                                    onUpdate: function (number, element) {
                                        element.innerHTML = Math.ceil(number.value);
                                    }
                                });
                            }
                    };
                    var childParams = this.getChildParams(element);
                    if(childParams){
                        var childs = Array.from(element.children);
                        if(childParams.params.reverse){
                            delete childParams.params.reverse;
                            childs = childs.reverse(0);
                        }
                        window.TweenMax.staggerTo(childs, childParams.duration, childParams.params, childParams.stagger);
                    };
                };
            };
        }.bind(this);
        // getParams
        this.getParams = function(element){
            var data = [];
            var index = 1;
            while(index > 0){
                var paramsStr;
                var attr = "data-a-to-"+index;
                if(index == 1) paramsStr = element.getAttribute("data-a-to");
                else paramsStr = element.getAttribute(attr);
                if(!paramsStr){ index = 0; break; }
                paramsStr = cuppa.replace(paramsStr, ";", ",");
                params = paramsStr.split(",");
                var tween = {duration:0.6, params:{}};
                for(var j = 0; j < params.length; j++){
                    var tmp2 = params[j].split(":");
                    var key = cuppa.trim(tmp2[0]);
                    var value = cuppa.trim(tmp2[1]);
                    if(key == "duration") tween.duration = cuppa.parseFloat(value);
                    else if(key == "numberFrom") tween.numberFrom = cuppa.parseFloat(value);
                    else if(key == "numberTo") tween.numberTo = cuppa.parseFloat(value);
                    else if(key == "number") tween.numberTo = cuppa.parseFloat(value);
                    else if(value == "true") tween.params[key] = true;
                    else if(value == "false") tween.params[key] = false;
                    else if(isNaN(value)) tween.params[key] = value;
                    else if(key && value != undefined) tween.params[key] = cuppa.parseFloat(value);
                };
                data.push(tween);
                index++;
            };
            return data;
        }.bind(this);
        // getAttrData
        this.getAttrData = function(element, attr){
            var opts = {};
            var attr = cuppa.attr(element, "data-a-opts");
                if(!attr) return opts;
                attr = attr.split(",");
            for(var j = 0; j < attr.length; j++){
                try{
                    var tmp2 = attr[j].split(":");
                    var key = cuppa.trim(tmp2[0]);
                    var value = cuppa.trim(tmp2[1]);
                    if(value == "true") opts[key] = true;
                    else if(value == "false") opts[key] = false;
                    else if(isNaN(value)) opts[key] = value;
                    else if(key && value != undefined) opts[key] = cuppa.parseFloat(value);
                }catch(err){ }
            };
            return opts;
        }.bind(this);
        // getChild Params
        this.getChildParams = function(element){
            var params = element.getAttribute("data-a-stagger-child");
            if(!params) return;
                params = params.split(",");
            var tween = {duration:0.6, params:{}, stagger:0.2};
            for(var j = 0; j < params.length; j++){
                var tmp2 = params[j].split(":");
                var key = cuppa.trim(tmp2[0]);
                var value = cuppa.trim(tmp2[1]);
                if(key == "duration") tween.duration = cuppa.parseFloat(value);
                else if(key == "stagger") tween.stagger = cuppa.parseFloat(value);
                if(value == "true") tween.params[key] = true;
                else if(value == "false") tween.params[key] = false;
                else if(isNaN(value)) tween.params[key] = value;
                else if(key && value != undefined) tween.params[key] = cuppa.parseFloat(value);
            };
            return tween;
        }.bind(this);
        cuppa.scroll(this.scroll, this.opts.scrollElement); this.scroll();
        // windowLoaded
        this.onLoaded = function(e){
            window.TweenMax.killTweensOf(this.opts.loadExpire);
            cuppa.off(window, "load", this.onLoaded);
            this.windowLoaded = true;
            this.scroll();
        }.bind(this);
        cuppa.on(window, "load", this.onLoaded);
        if(this.opts.loadExpire) window.TweenMax.delayedCall(this.opts.loadExpire, this.onLoaded);
    };
/*
    tween mouse

    class='cu-tween-mouse'
    attributes
        data-a-rollover='y:-10, duration:0.8'
        data-a-rollout='y:0, duration:0.8'
        data-a-click='y:-5, duration:0.8'
        * To apply effect to child elements, add same attributes to child elements

 */
    cuppa.tweenMouse = function(elements){
        this.elements = elements || ".cu-tweenmax-mouse, .cu-tween-mouse";
        this.elements = cuppa.element(this.elements);

        this.onRollOver = function(e){
            var attr = "data-a-rollover";
            var element = e.currentTarget;
            var childs = cuppa.element("["+attr+"]", {target:element});
            this.processAnimation([element], attr);
            this.processAnimation(childs, attr);
        }.bind(this);

        this.onRollOut = function(e){
            var attr = "data-a-rollout";
            var element = e.currentTarget;
            var childs = cuppa.element("["+attr+"]", {target:element});
            this.processAnimation([element], attr);
            this.processAnimation(childs, attr);
        }.bind(this);

        this.onClick = function(e){
            var attr = "data-a-click";
            var element = e.currentTarget;
            var childs = cuppa.element("["+attr+"]", {target:element});
            this.processAnimation([element], attr);
            this.processAnimation(childs, attr);
        }.bind(this);

        this.processAnimation = function(items, attr, duration){
            if(!Array.isArray(items)) items = [items];
            for(var i = 0; i < items.length; i++){
                var item = items[i];
                var data = cuppa.getTweenMaxParams(item, attr);
                cuppa.animateTweenMaxByAttrs(item, data, duration);
            }
        }.bind(this);

        // init
        for(var i = 0; i < this.elements.length; i++){
            var element = this.elements[i];
            cuppa.on(element, "mouseenter", this.onRollOver, "window.TweenMaxButton");
            cuppa.on(element, "mouseleave", this.onRollOut, "window.TweenMaxButton");
            cuppa.on(element, "mousedown", this.onClick, "window.TweenMaxButton");
            cuppa.on(element, "mouseup", this.onRollOver, "window.TweenMaxButton");

            var attr = "data-a-rollout";
            var childs = cuppa.element("["+attr+"]", {target:element});
            this.processAnimation(element, attr, 0);
            this.processAnimation(childs, attr, 0);
        };
    };
    cuppa.getTweenMaxParams = function(element, attribute){
        var data = [];
        var index = 1;
        while(index > 0){
            var paramsStr;
            var attr = attribute+"-"+index;
            if(index == 1) paramsStr = element.getAttribute(attribute);
            else paramsStr = element.getAttribute(attr);
            if(!paramsStr){ index = 0; break; }
            paramsStr = cuppa.replace(paramsStr, ";", ",");
            var params = paramsStr.split(",");
            var tween = {duration:0.3, params:{}};
            for(var j = 0; j < params.length; j++){
                var tmp2 = params[j].split(":");
                var key = cuppa.trim(tmp2[0]);
                var value = cuppa.trim(tmp2[1]);
                if(key == "duration") tween.duration = cuppa.parseFloat(value);
                else if(key == "numberFrom") tween.numberFrom = cuppa.parseFloat(value);
                else if(key == "numberTo") tween.numberTo = cuppa.parseFloat(value);
                else if(key == "number") tween.numberTo = cuppa.parseFloat(value);
                else if(value == "true") tween.params[key] = true;
                else if(value == "false") tween.params[key] = false;
                else if(isNaN(value)) tween.params[key] = value;
                else if(key && value != undefined) tween.params[key] = cuppa.parseFloat(value);
            };
            data.push(tween);
            index++;
        };
        return data;
    };
    cuppa.animateTweenMaxByAttrs = function(element, params, duration){
        window.TweenMax.killTweensOf(element);
        element.timeline = new window.TimelineMax();
        element.timelineNumber = new window.TimelineMax();
        for(var j = 0; j < params.length; j++){
            var duration = (duration != undefined && duration != null) ? duration : params[j].duration;
            element.timeline.to(element, duration, params[j].params);
            // animate numbers
            if(params[j].numberTo){
                var n1 = { value: params[j].numberFrom || parseInt(element.innerHTML) };
                element.timelineNumber.to(n1, duration, {value: params[j].numberTo, delay:params[j].params.delay, ease:params[j].params.ease, onUpdateParams:[n1, element],
                    onUpdate: function (number, element) {
                        element.innerHTML = Math.ceil(number.value);
                    }
                });
            }
        };
    }.bind(this);
/* table fixed */
    cuppa.tableFixed = function(table, opts){
        this.opts = cuppa.mergeObjects([{class:"table-sticky"},opts]);
        this.bodyTrs = cuppa.elements("tr", {parent:table});
        this.bodyTds = cuppa.elements("tbody tr td."+this.opts.class, {parent:table});
        this.headTrs = cuppa.elements("thead tr", {parent:table});
        this.headThs = cuppa.elements("thead tr th."+this.opts.class, {parent:table});
        this.parent = cuppa.parent(table);


        this.onScroll = function(){
            let scroll = cuppa.scrollPosition(this.parent);
            cuppa.css(this.bodyTds, {"margin-left":scroll.x+"px"});
            cuppa.css(this.headThs, {"margin-left":scroll.x+"px"});
        }.bind(this); cuppa.scroll(this.onScroll, this.parent);

        this.analyseHeadTr = function(tr){
            let ths = cuppa.elements("th."+this.opts.class, {parent:tr});
            for(let j = 0; j <= ths.length; j++){
                let th = ths[j]; if(!th) return;
                let dim = cuppa.dim(th);
                cuppa.css(th, {position:"absolute", width:dim.width+"px", height:dim.height+"px", left:dim.x2+"px", "z-index":2, opacity:1});
                if(j == ths.length-1){
                    let fakeTh = cuppa.newElement("<th></th>");
                    cuppa.css(fakeTh, {opacity:1, width:dim.width+"px", background:"#F0F"})
                    cuppa.after(fakeTh, th);
                }
            }
        }.bind(this);

        this.anylizeBodyTr = function(tr){
            let tds = cuppa.elements("td."+this.opts.class, {parent:tr});
            if(!tds.length) return;
            for(let j = 0; j <= tds.length; j++){
                let td = tds[j]; if(!td) return;
                let dim = cuppa.dim(td);
                cuppa.css(td, {position:"absolute", width:dim.width+"px", height:dim.height+"px", left:dim.x2+"px", "z-index":1});
                if(j == tds.length-1){
                    let fakeTd = cuppa.newElement("<td></td>");
                    cuppa.css(fakeTd, {opacity:0});
                    cuppa.after(fakeTd, td);
                }
            }
        }.bind(this);

        for(let i = 0; i < this.bodyTrs.length; i++){
            this.anylizeBodyTr(this.bodyTrs[i]);
        }

        for(let i = 0; i < this.headTrs.length; i++){
            this.analyseHeadTr(this.headTrs[i]);
        }
    };
/* Packery
    Example:  new cuppa.packery();
    <div class="cu-packery">
        div class="item" style="width:50%; height:400px;"></div>
        <div class="item" style="width:25%; height:200px;"></div>
        <div class="item" style="width:25%; height:200px;"></div>
    </div>
*/
    cuppa.packery = function(elements, opts){
        if(this.elements == undefined) this.elements = ".cu-packery";
        this.elements = cuppa.element(this.elements);
        this.opts = opts || {};
        this.opts.itemSelector = this.opts.itemSelecto || '.item';
        this.opts.transitionDuration = this.opts.transitionDuration || 0;
        this.ajust = function(){
            for(var i = 0; i < this.elements.length; i++){
                var element = this.elements[i];
                var pckry = new window.Packery(element, this.opts);
            };
        }.bind(this); this.ajust();
        cuppa.on(window, "resize", this.ajust);
    };
/* layers
    opts = {
        duration:0.2
        target:window
    }
*/
    cuppa.layer = function(elements, opts){
        this.opts = opts || {};
        this.opts.duration = this.opts.duration || 0.2;
        this.opts.target =this.opts.target || document;
        //header1 wrapp
        this.elements = elements;
        if(!this.elements) this.elements = document.getElementsByClassName("cu-layer");
        for(var i = 0; i < this.elements.length; i++){
            var element = this.elements[i];
            var xRange = element.getAttribute("data-x-range");
            var yRange = element.getAttribute("data-y-range");
            var xRotate = element.getAttribute("data-x-rotate");
            var yRotate = element.getAttribute("data-y-rotate");
            if(xRange){
                element.xRange = xRange.split(",");
                    element.xRange[0] = parseFloat(element.xRange[0]);
                    element.xRange[1] = parseFloat(element.xRange[1]);
            }
            if(yRange){
                element.yRange = yRange.split(",");
                element.yRange[0] = parseFloat(element.yRange[0]);
                element.yRange[1] = parseFloat(element.yRange[1]);
            }
            if(xRotate){
                element.xRotate = xRotate.split(",");
                element.xRotate[0] = parseFloat(element.xRotate[0]);
                element.xRotate[1] = parseFloat(element.xRotate[1]);
            }
            if(yRotate){
                element.yRotate = yRotate.split(",");
                element.yRotate[0] = parseFloat(element.yRotate[0]);
                element.yRotate[1] = parseFloat(element.yRotate[1]);
            }
        };
        this.mouseMove = function(e){
            var percX = cuppa.percent(e.screenX, 0, cuppa.width());
            var percY = cuppa.percent(e.screenY, 0, cuppa.height());
            for(var i = 0; i < this.elements.length; i++){
                var element = this.elements[i];
                if(element.xRange){
                    var posX = cuppa.random(element.xRange[0], element.xRange[1], percX);
                    window.TweenMax.to(element, this.opts.duration, {x:posX});
                }
                if(element.yRange){
                    var posY = cuppa.random(element.yRange[0], element.yRange[1], percY);
                    window.TweenMax.to(element, this.opts.duration, {y:posY});
                }
                if(element.xRotate){
                    var value = cuppa.random(element.xRotate[0], element.xRotate[1], percX);
                    window.TweenMax.set(element, {transformPerspective:600})
                    window.TweenMax.to(element, this.opts.duration, {rotationY:value});
                }
                if(element.yRotate){
                    var value = cuppa.random(element.yRotate[0], element.yRotate[1], percY);
                    window.TweenMax.set(element, {transformPerspective:600});
                    window.TweenMax.to(element, this.opts.duration, {rotationX:value});
                }
            }
        }.bind(this);
        this.opts.target.addEventListener("mousemove", this.mouseMove);
    };
/* responsive image */
    cuppa.responsiveImage = function(elements, opts){
        this.opts = cuppa.mergeObjects([{property:"width"  }, opts]);
        if(elements == undefined) elements = "img.responsive";
        this.elements = cuppa.element(elements);
        this.onLoad = function(element){
            if(element.currentTarget) element = element.currentTarget;
            var noresponsive = ( cuppa.hasClass(element, "noresponsive") || cuppa.hasClass(element, "no-responsive") );
            if(noresponsive) return;
            var dim = cuppa.dim(element);
            if(this.opts.property == "width"){
                var width = dim.width;
                if(!width) width = element.naturalWidth;
                if(width){
                    cuppa.css(element, {width:"100%", "max-width":width+"px", height:"auto"});
                }else{

                }
            };
        }.bind(this);
        cuppa.on(this.elements, "load", this.onLoad);
        this.elements.forEach(this.onLoad);

        //jQuery(element).each(onLoad);
        //jQuery(window).load(function(){ jQuery(element).each(onLoad); });
    };
/* ParallaxBg
    <div class="parallax_bg" style="background-image:url(images/bg.jpg)" >
    new cuppa.parallaxBg('.parallax_bg');
    opts = {}
    opts.invert = false;
*/
    cuppa.parallaxBackground = function(elements, opts){
        if(elements == undefined) elements = ".parallax-bg";
        elements = cuppa.element(elements); if(!elements.length) return;
        this.opts = opts || {};
        this.opts.invert = this.opts.invert || false;
        this.items = [];
        for(var i = 0; i < elements.length; i++){
            if(elements[i].style.backgroundImage){
                var item = {}
                    item.element = elements[i];
                    item.url = elements[i].style.backgroundImage;
                        item.url = cuppa.replace(item.url, 'url\\("', '');
                        item.url = cuppa.replace(item.url, '"\\)', '');
                        item.url = cuppa.replace(item.url, 'url\\(', '');
                        item.url = cuppa.replace(item.url, '\\)', '');
                    item.img = cuppa.newElement("<img src='"+item.url+"' class='parallax-bg-img no-responsive' style='display:none; position: absolute; top: 0px; left: 0px; width: 100%; height: auto;' ></img>");
                    cuppa.prepend(item.img,item.element);
                    item.element.style.overflow = "hidden";
                this.items.push(item);
            }
        }

        this.onScroll = function(){
            for(var i = 0; i < this.items.length; i++){
                var item = this.items[i];
                var dim = cuppa.dim(item.element);
                var percent = cuppa.percent(dim.y3, -dim.height, window.innerHeight, this.opts.invert);
                if(percent > 1) percent = 1; if(percent < 0) percent = 0;
                var img = item.img;
                var dim_img = cuppa.dim(img);
                var pos = -percent*(dim_img.height - dim.height);
                if(pos > 0) pos = 0;
                img.style.top = pos + "px";
            }
        }.bind(this);

        this.resize = function(e){
            for(var i = 0; i < this.items.length; i++){
                var item = this.items[i];
                var dim_ele = cuppa.dim(item.element);
                var dim_img = cuppa.dim(item.img);
                if(dim_ele.height >= dim_img.height-20){
                    window.TweenMax.set(item.img, {display:"none"});
                }else{
                    window.TweenMax.set(item.img, {display:"block"});
                    this.onScroll();
                }
            }
        }.bind(this);
        cuppa.scroll(this.onScroll); this.onScroll();
        window.addEventListener("load", this.onScroll, true);
        window.addEventListener("resize", this.resize);
        window.addEventListener("load", this.resize, true);
        this.resize();
        return this;
    };
    cuppa.parallaxBg = function(elements, options){ new cuppa.parallaxBackground(elements, options); };
// parallax
    cuppa.parallax = function(elements, opts){
        this.opts = cuppa.mergeObjects([{}, opts]);
        if(!elements) elements = document.getElementsByClassName("cu-parallax");
        this.elements = cuppa.element(elements);
        // constructor
            this.constructor = function(){
                for(var i = 0; i < this.elements.length; i++){
                    var element = this.elements[i];
                        element.parallaxTween = new window.TimelineMax();
                        var initParams = this.getParams(element, "data-p-from");
                        if(initParams) element.parallaxTween.set(element, initParams.params);
                        element.parallaxTween.stop();
                        // set other tweeens
                            var data = this.getTweens(element);
                            if(data){
                                for(var j = 0; j < data.length; j++){
                                    var tween = data[j];
                                    element.parallaxTween.to(element, tween.duration, tween.params);
                                };
                            };
                };
                cuppa.scroll(this.onScroll); this.onScroll();
            }.bind(this);
        // onScroll
            this.onScroll = function(element){
                for(var i = 0; i < this.elements.length; i++){
                    this.update(this.elements[i]);
                };
            }.bind(this);
        // update
            this.update = function(element){
                var rect = element.getBoundingClientRect();
                var viewHeight = Math.max(document.documentElement.clientHeight, window.innerHeight);
                var percent = cuppa.percent(rect.top + rect.height*0.5, 0-rect.height*0.5, viewHeight+rect.height*0.5, true);
                element.parallaxTween.seek(percent*element.parallaxTween.duration());
            }.bind(this);
        // getData
            this.getTweens = function(element){
                var data = [];
                var index = 1;
                while(index > 0){
                    var paramsStr;
                    var attr = "data-p-to-"+index;
                    if(index == 1) attr = "data-p-to";
                    var tween = this.getParams(element, attr);
                    if(!tween){ index = 0; break; }
                    data.push(tween);
                    index++;
                };
                return data;
            }.bind(this);
        // getParams
            this.getParams = function(element, attr){
                var paramsStr = element.getAttribute(attr);
                if(!paramsStr) return;
                paramsStr = cuppa.replace(paramsStr, ";", ",");
                var attr = attr;
                paramsStr = element.getAttribute(attr);
                var params = paramsStr.split(",");
                var tween = {duration:0.6, params:{ } };
                for(var j = 0; j < params.length; j++){
                    var tmp2 = params[j].split(":");
                    var key = cuppa.trim(tmp2[0]);
                    var value = cuppa.trim(tmp2[1]);
                    if(key == "duration") tween.duration = cuppa.parseFloat(value);
                    else if(key == "numberFrom") tween.numberFrom = cuppa.parseFloat(value);
                    else if(key == "numberTo") tween.numberTo = cuppa.parseFloat(value);
                    else if(key == "number") tween.numberTo = cuppa.parseFloat(value);
                    else if(value == "true") opts[key] = true;
                    else if(value == "false") opts[key] = false;
                    else if(isNaN(value)) tween.params[key] = value;
                    else if(key && value != undefined) tween.params[key] = cuppa.parseFloat(value);
                };
                return tween;
            }.bind(this);
        // start
            this.constructor();
    };
// css
    /* Set / Get
        set
            element = element || array of elements
        get
            element = element
            opts = {}
            opts.number = false; // true: force to return number
    */
    cuppa.css = function(elements, property, opts){
        if(!elements) return;
        elements = cuppa.element(elements);
        for(var k = 0; k < elements.length; k++){
            var element = elements[k];
            if(typeof(property) == "object"){
                if(!Array.isArray(element)) element = [element];
                for(var i = 0; i < element.length; i++){
                    for (var css in property) {
                        var value = property[css];
                        var priority = "";
                        if(value){
                            value = String(value);
                            if(value.indexOf("!important") != -1){
                                priority = "important";
                                value = value.replace("!important", "");
                            };
                            element[i].style.setProperty( cuppa.trim(css), cuppa.trim(value), priority);
                        };
                    };
                };
            }else{
                // opts default
                    opts = opts || {};
                    opts.number = opts.number || false;
                // get value
                    try{
                        var style = window.getComputedStyle(element);
                        value = style.getPropertyValue(property);
                        if(opts.number) value = parseFloat(value) || 0;
                    }catch(err){ value = null; };
                return value;
            };
        };
    };
/* sumCss */
    cuppa.cssSum = function(elements, property, includeMargin){
        if(includeMargin == undefined) includeMargin = true;
        elements = cuppa.element(elements);
        var result = 0;
        for(var i = 0; i < elements.length; i++){
            var element = elements[i];
            if(property == "height" || property == "width"){
                var dim = cuppa.dim(element);
                if(includeMargin) result += dim[property+"4"];
                else result += dim[property];
            }else{
                result += cuppa.css(element, property, {number:true});
            }
        };
        return result;
    };
/* maxIndex */
    cuppa.maxZIndex = function(allElements){
        var zIndex = 1;
        var childs;
        if(allElements == undefined) allElements = true;
        if(allElements) childs = cuppa.element("*");
        for(var i = 0; i <  childs.length; i++){
            var child = childs[i];
            var style = getComputedStyle(child);
            var zIndexTemp = cuppa.parseFloat(style.zIndex);
            if(zIndexTemp > zIndex) zIndex = zIndexTemp;
        }
        return zIndex;
    };
/* blur */
    cuppa.blur = function(){
        if (document.activeElement != document.body) document.activeElement.blur();
        cuppa.unselect();
    };
// focus
    cuppa.focus = function(element){ element.focus(); };
/* unselect */
    cuppa.unselect = function(){
        if (document.selection) {
            document.selection.empty();
        }else if( window.getSelection) {
            window.getSelection().removeAllRanges();
        };
    };
/* mergeObjects, create a new obj with the values of objs in Array.
    If create_new_object = true, create a new Oject an Add all element to it, else join to the first object all elements
    create_new_object = false
*/
    cuppa.mergeObjects = function(array_objs, create_new_object){
        if(!create_new_object){
            var obj1 = array_objs.shift();
            for(var i = 0; i < array_objs.length; i++){
                var obj = array_objs[i];
                if(obj){ for (var attrname in obj) { obj1[attrname] =  obj[attrname]; } }
            };
            return obj1;
        }else{
            var tmp_obj = {};
            for(var i = 0; i < array_objs.length; i++){
                var obj = array_objs[i];
                if(obj){ for (var attrname in obj) { tmp_obj[attrname] = obj[attrname]; } }
            };
            return tmp_obj;
        };
    }; cuppa.joinObjects = function(array_objs, create_new_object){ return cuppa.mergeObjects(array_objs, create_new_object); };

// mergeArray, create a new array with the values all array passed.
    cuppa.mergeArrays = function(arrays){
        var result = [];
        for(var i = 0; i < arrays.length; i++){
            if(arrays[i]){
                result = result.concat(arrays[i]);
            };
        };
        return result;
    }; cuppa.joinArrays = function(arrays){ return cuppa.mergeArrays(arrays); };

/* is Mobile */
    cuppa.isMobile = function() {
        var check = false;
        (function(a){if(/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i.test(a)||/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0,4))) check = true;})(navigator.userAgent||navigator.vendor||window.opera);
        return check;
    };
/* flash support */
    cuppa.flashSupport = function(){
        var hasFlash = false;
        try { if(new window.ActiveXObject('ShockwaveFlash.ShockwaveFlash')) hasFlash = true; } catch (e) { };
        if(!hasFlash){
            if (navigator.mimeTypes && navigator.mimeTypes['application/x-shockwave-flash'] != undefined && navigator.mimeTypes['application/x-shockwave-flash'].enabledPlugin) {
                hasFlash = true;
            };
        };
        return hasFlash;
    };
/* WheelSmooth
    var scroll = cuppa.wheelSmooth(target);
        scroll.destroy();

    If is used in Body, add the next CSS: body, html{ height: 100%; }
*/
    cuppa.wheelSmooth = function(target, options){
        if(target == undefined) target = document.body;
        // opts
            this.options = options || {};
            this.options.scrollingCallback = this.options.scrollingCallback || null;
            this.options.fricton = this.options.fricton || 0.9;
            this.options.vy = this.options.vy || 0;
            this.options.stepAmt = this.options.stepAmt || 3;
            this.options.minMovement = this.options.minMovement || 0.1;
            this.options.ts = this.options.ts || 0.1;
            this.target = cuppa.element(target, {returnType:"first"});
        // variables
            this.target = target;
            this.container = target;
            this.running = false;
            this.currentY = 0;
            this.targetY = 0;
            this.oldY = 0;
            this.maxScrollTop = 0;
            this.minScrollTop = 0;
            this.direction = 0;
        // methods;
            this.updateScrollTarget = function (amt) {
                this.targetY += amt;
                this.options.vy += (this.targetY - this.oldY) * this.options.stepAmt;
                this.oldY = this.targetY;
            }.bind(this);

            this.render = function () {
                if (this.options.vy < -(this.options.minMovement) || this.options.vy > this.options.minMovement) {
                    this.currentY = (this.currentY + this.options.vy);
                    if (this.currentY > this.maxScrollTop) {
                        this.currentY = this.options.vy = 0;
                    } else if (this.currentY < this.minScrollTop) {
                        this.options.vy = 0;
                        this.currentY = this.minScrollTop;
                    }
                    cuppa.scrollMove(this.target, {y:-this.currentY, duration:0});
                    this.options.vy *= this.options.fricton;
                    if(this.options.scrollingCallback){ this.options.scrollingCallback(this); }
                }
            }.bind(this);

            this.animateLoop = function () {
                if(!this.running) return;
                cuppa.requestAnimationFrame(this.animateLoop);
                this.render();
            }.bind(this)

            this.onUpdateSize = function () {
                if(this.container.tagName == "BODY") this.container = document.querySelector("html");
                var dim = cuppa.dim(this.container);
                this.minScrollTop = dim.clientHeight - dim.scrollHeight;
            }.bind(this);

            this.onWheel = function (e) {
                this.running = true;
                e.preventDefault();  e.stopPropagation();
                var evt = e.originalEvent;
                this.onUpdateSize();
                var delta = 0;
                    if(e.detail) delta = -1*e.detail;
                    if(e.deltaY) delta = 1*e.deltaY/40;
                    if(e.wheelDelta) delta = 1*e.wheelDelta/40;
                var dir = delta < 0 ? -1 : 1;
                if (dir != this.direction) {
                    this.options.vy = 0;
                    this.direction = dir;
                }
                this.currentY = -cuppa.scrollPosition(this.target).y;
                this.delta = delta;
                this.updateScrollTarget(delta);
            }.bind(this);

            this.destroy = function(){
                this.container.removeEventListener("mousewheel", this.onWheel);
                this.container.removeEventListener("DOMMouseScroll", this.onWheel);
                window.removeEventListener("resize", this.onUpdateSize);
            }.bind(this);

            this.addEvents = function(){
                 this.container.addEventListener("mousewheel", this.onWheel);
                 this.container.addEventListener("DOMMouseScroll", this.onWheel);
                 window.addEventListener("resize", this.onUpdateSize);
            }.bind(this);

        // init
            this.destroy(); this.addEvents(); this.onUpdateSize();
            var pos = cuppa.scrollPosition(this.container);
            this.targetY = this.oldY = pos.y; this.currentY = -this.targetY;
            if(this.options.scrollingCallback) this.options.scrollingCallback();
            if(!this.running){ this.running = true; this.animateLoop(); }
        return this;
    };
    cuppa.wheelSmoothDisable = function(target){
        if(target == undefined) target = document.body;
        //$(target).smoothWheelRemove();
    };
/* requestAnimationFrame */
    cuppa.requestAnimationFrame = function (callback) {
        window.requestAnimationFrame(callback) ||
        window.webkitRequestAnimationFrame(callback) ||
        window.mozRequestAnimationFrame(callback) ||
        window.oRequestAnimationFrame(callback) ||
        window.msRequestAnimationFrame(callback) ||
        window.setTimeout(callback, 1000 / 60);
    };
/* Set Cookie, By default, the cookie is deleted when the browser is closed */
	cuppa.setCookie = function(name, value, exdays) {
	   if(exdays){
            var exdate = new Date();
                exdate.setDate(exdate.getDate() + exdays);
            var value = escape(value) + ((exdays==null) ? "" : "; expires="+exdate.toUTCString());
            document.cookie = name + "=" + value + ";path=/";
        }else{
            document.cookie = name + "=" + value + ";path=/";
        }
	};
/* Get Cookie */
    cuppa.getCookie = function(name, documentRef) {
        if(documentRef == undefined) documentRef = document;
        var results = documentRef.cookie.match ( '(^|;) ?' + name + '=([^;]*)(;|$)' );
        if ( results ) return ( unescape ( results[2] ) );
        else return null;
    };
/* Get Cookie Path */
    cuppa.getCookiePath = function(names, path, callback){
        if(typeof names == "string") names = [names];
        if(!document){ callback({}); return; }
        let iframe = document.createElement("iframe");
            iframe.style.position = "fixed"; iframe.style.top = 0; iframe.style.left = 0; iframe.style.width = 0; iframe.style.height = 0; iframe.style.opacity = 0;
            iframe.setAttribute("src", path);
            iframe.onload = function(){
                let documentRef = iframe.contentWindow.document;
                let result = {};
                for(let i = 0; i < names.length; i++){
                    let name = names[i];
                    result[name] = cuppa.getCookie(name, documentRef);
                }
                document.body.removeChild(iframe);
                if(callback) callback(result);
            };
        document.body.append(iframe);
    }
/* Delete Cookie*/
	cuppa.deleteCookie = function(name){
        document.cookie = name+"=; expires=Thu, 01 Jan 1970 00:00:00 UTC; path=/;";
	};
 // local / session storage
    cuppa.sessionStorage = function(name, value, cookie){
        if(cookie == undefined) cookie = false;
        if(value == undefined){
            return sessionStorage.getItem(name);
        }else{
            sessionStorage.setItem(name, value);
            if(cookie) cuppa.setCookie(name, value);
        };
    };
    cuppa.localStorage = function(name, value, cookie){
        if(cookie == undefined) cookie = false;
        if(value == undefined){
            return localStorage.getItem(name);
        }else{
            localStorage.setItem(name, value);
            if(cookie) cuppa.setCookie(name, value);
        };
    };
/* Config URL Friendly HTML5
    setParams params: { path:string, title:string, data:object }
 */
    cuppa.URLHash = '';
    cuppa.URLBase = '';
    cuppa.getPath = function(returnArray, opts){
        opts = cuppa.mergeObjects([{removeBase:true}, opts]);
        if(!document) return;
        var basePath = document.querySelector("base");
        var path = window.location.href;
        if(basePath && basePath.getAttribute("href") != "/" && opts.removeBase) path = cuppa.replace(path, basePath.getAttribute("href"), "");
        if(cuppa.URLBase) path = cuppa.replace(path, cuppa.URLBase, "");
        path = cuppa.replace(path,"//","");
        path = cuppa.replace(path,"https:","");
        path = cuppa.replace(path,"http:","");
        if(!basePath && !cuppa.URLBase){
            try{ path = path.substr(path.indexOf("/")+1); }catch(err){ }
        }
        if(returnArray) path = path.split("/");
        return path;
    };
    cuppa.setPath = function(path, title, callback){
        if(title == undefined) title = "";
        if(path == undefined || path == "") path = " ";
        else path = cuppa.URLHash+path;
        window.history.pushState(path, title, path);
        if(callback) callback();
    };
    cuppa.back = function(){
        window.history.back();
    };
    cuppa.forward = function(){
        window.history.forward();
    };
    cuppa.getPathData = function(path, opts){
        if(!path) path = cuppa.getPath();
        path = cuppa.replace(path, cuppa.URLHash, "");
        opts = cuppa.mergeObjects([{removeFirst:false}, opts]);
        var obj = {url:path};
        // get base
            var base = path;
                base = cuppa.replace(base, "#/", ""); base = cuppa.replace(base, "#", "");
            if(base.indexOf("?") != -1) base = base.substr(0, base.indexOf("?"));
            obj.base = base;
            obj.baseArray = base.split("/");
            // remove first
                if(opts.removeFirst){
                    obj.baseArray.shift();
                    obj.base = obj.baseArray.join("/");
                }
        // domain
            obj.domain = cuppa.replace(obj.base,"https://","");
            obj.domain = cuppa.replace(obj.domain,"http://","");
            obj.domain = obj.domain.substr(0, obj.domain.indexOf("/"));
        // protocol
            obj.protocol = "http";
            if(path.indexOf("https://") != -1) obj.protocol = "https";
        // get data
            var dataStr = path;
            if(dataStr.indexOf("?") != -1 || dataStr.indexOf("#") != -1){
                if(dataStr.indexOf("?") != -1) dataStr = dataStr.substr(path.indexOf("?")+1);
                if(dataStr.indexOf("#") != -1) dataStr = dataStr.substr(path.indexOf("#")+1);
                var dataArray = dataStr.split("&");
                var data = {};
                for(var i = 0; i < dataArray.length; i++){
                    var parts = dataArray[i].split("=");
                    if(parts[0]) data[parts[0]] = parts[1] || '';
                };
                obj.data = data;
            }else{ obj.data = {}; };
        return obj;
    };
    cuppa.pathData = function(path, opts){ return cuppa.getPathData(path, opts); };
/* arrayOfValues
    opts:{
        noValueBehaviour:end, remove
    }
*/
    cuppa.extractArray = function(arrayData, column, opts){
        opts = cuppa.mergeObjects([{removeDuplicates:true, orderBy:"ASC", noValueBehaviour:'end'}, opts]);
        let result = arrayData.map(row=>row[column]);
        if(opts.removeDuplicates) result = Array.from(new Set(result));
        if(opts.orderBy.toLowerCase() == "asc") result = result.sort();
        if(opts.orderBy.toLowerCase() == "desc"){ result = result.sort(); result = result.reverse(); }
        if(opts.noValueBehaviour){
            var index = result.indexOf("");
            if(index != -1 && opts.noValueBehaviour == "end"){
                result.splice(index,1);
                result.push("");
            }else if(index != -1 && opts.noValueBehaviour == "remove"){
                result.splice(index,1);
            }
        }
        return result;
    }
/* sumArray */
    cuppa.sumArray = function(data){
        var total = 0;
        for(var i = 0; i < data.length; i++){ total += parseFloat(data[i]); }
        return total;
    };
// part of Number
    cuppa.numberParts = function(number){
        var parts = String(number).split(".");
        parts[0] = (parts[0]) ? parseFloat(parts[0]): 0;
        parts[1] = (parts[1]) ? parseFloat("."+parts[1]): 0;
        return parts;
    };
// cuppa filter
    cuppa.filterArray = function(array, opts){
        opts = cuppa.mergeObjects([{conds:null, type:"OR"}, opts]);
        if(!array) array = [];
        var result = [];
        for(let i = 0; i < array.length; i++){
            let row = array[i];
            let valid = false;
            if(opts.conds){
                for(let j = 0; j < opts.conds.length; j++) {
                    valid = cuppa.filterArrayValidate(row, opts.conds[j]);
                    if(valid) break;
                }
            }
            if(valid) result.push(row);
        }
        return result;
    };
    cuppa.filterArrayValidate = function(row, condStr, opts){
        opts = cuppa.mergeObjects([{caseSensitive:true}, opts]);
        let condParts;
        let cond = {};

        if(condStr.indexOf(">=") != -1){ condParts = condStr.split(">="); cond.condition = ">="; }
        else if(condStr.indexOf("<=") != -1){ condParts = condStr.split("<="); cond.condition = "<="; }
        else if(condStr.indexOf("==") != -1){ condParts = condStr.split("=="); cond.condition = "=="; }
        else if(condStr.indexOf("like") != -1){ condParts = condStr.split("like"); cond.condition = "like"; }
        else if(condStr.indexOf("LIKE") != -1){ condParts = condStr.split("LIKE"); cond.condition = "LIKE"; }

        cond.column = cuppa.trim(condParts[0]);
        cond.value = row[cond.column];
        cond.compareWith = cuppa.trim(condParts[1]);

        if(opts.caseSensitive){ cond.value = cond.value.toLowerCase(); cond.compareWith = cond.compareWith.toLowerCase(); }

        if(cond.condition == ">="){
            if(cond.value >= cond.compareWith) return true;
        }else if(cond.condition == "<="){
            if(cond.value  <= cond.compareWith) return true;
        }else if(cond.condition == "=="){
            if(cond.value == cond.compareWith) return true;
        }else if(cond.condition.toLowerCase() == "like"){
            if(cuppa.likeString(cond.value, cond.compareWith)) return true;
        }
        return false;
    }
/* managerURL
*   new cuppa.managerURL(opts);
*   opts = {
*       autoInit:true;
*       updateLinks:true;
*       callback:Function
*       hash:''
*   }
* */
    cuppa.managerURL = new function(opts){
        this.opts = cuppa.mergeObjects([{updateLinks:false, autoInit:true}, opts]);
        this.titles = {};
        this.path = "";
        this.pathArray = null;
        this.callbacks = [];
        // setPath
            this.setPath = function(path, title){
                if(path != undefined) cuppa.setPath(path, title);
                this.path = cuppa.getPath();
                this.pathArray = cuppa.getPath(true);
                if(title) document.title = title;
                if(this.opts.callback) this.opts.callback(this.path, this.pathArray);
                if(cuppa.dispatchEvent){
                    cuppa.dispatchEvent({ type: "URL_CHANGE", message: this });
                    cuppa.dispatchEvent({ type: "CHANGE_URL", message: this });
                }
                var pathToSend = this.path.replace(cuppa.URLHash, "");
                var pathToSendArray = (pathToSend) ? pathToSend.split("/") : [];
                for(var i = 0; i < this.callbacks.length; i++){
                    try{ this.callbacks[i](pathToSend, pathToSendArray); }catch(err){ }
                }
            }.bind(this);
            this.set = function(path, title){ this.setPath(path, title); }.bind(this);
        // Auto configure 'a' tags
            this.updateLinks = function(elements){
                if(!elements) elements = cuppa.element("a");
                cuppa.on(elements, "click", this.onClick);
            }.bind(this);
            this.update = function(elements){ this.updateLinks(elements); }.bind(this);

            this.onClick = function(e){
                var element = e.currentTarget;
                var href = element.getAttribute("href");
                var target = element.getAttribute("target");
                var title = element.getAttribute("title");
                if(href == undefined || href.search("http") != -1) return;
                if(target && target != "_self") return;
                e.preventDefault();
                this.setPath(href, title);
            }.bind(this);

            if(this.opts.updateLinks) this.updateLinks();
        // onHistory
            this.onHistory = function(e){
                this.path = e.state || "";
                if(typeof this.path != "string") return;
                this.pathArray = this.path.split("/");
                if(this.opts.callback) this.opts.callback(this.path, this.pathArray);
                cuppa.dispatchEvent({ type: "URL_CHANGE", message: this });
                cuppa.dispatchEvent({ type: "CHANGE_URL", message: this });
                var pathToSend = this.path.replace(cuppa.URLHash, "");
                var pathToSendArray = (pathToSend) ? pathToSend.split("/") : [];
                for(var i = 0; i < this.callbacks.length; i++){
                    try{ this.callbacks[i](pathToSend, pathToSendArray); }catch(err){ }
                }
            }.bind(this);
            cuppa.on(window, "popstate", this.onHistory);
        // addCallback
            this.addCallback = function(callback, autoInit){
                if(autoInit == undefined) autoInit = true;
                this.callbacks.push(callback);
                if(autoInit) this.setPath();
            }.bind(this);
        // init
            if(this.opts.autoInit) this.setPath();
        return this;
    }; cuppa.path = cuppa.managerURL;
/* ajax
    opts{
        data:Object
        method:"POST" // GET
        async: true // true,false
        header:{key:value, key:value}
        onError:callback
    }
*/
    cuppa.ajax = function(url, opts, callback){
        // opts
        opts = opts || { };
        opts.method = (opts.method) ? opts.method.toUpperCase() : "POST";
        if(opts.async == undefined) opts.async = true;
        var request = new XMLHttpRequest();
        request.open(opts.method, url, opts.async);
        //++ add header
            if(opts.headers) opts.header = opts.headers;
            if(!opts.header) opts.header = {};
            if(!opts.header["Content-Type"]) request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
            if(opts.header){
                for (var key in opts.header) {
                    request.setRequestHeader(key, opts.header[key]);
                };
            };
        //--
        request.addEventListener("load", onComplete);
        request.addEventListener("error", onError);
        if(opts.data && opts.header["Content-Type"] == "application/json") request.send(JSON.stringify(opts.data));
        else if(opts.data) request.send(cuppa.objectToURL(opts.data));
        else request.send();
        // onComplete
        function onComplete () {
            var result = this.responseText;
            if(opts.callback) opts.callback(result, this);
            if(callback) callback(result, this);
        };
        // onError
        function onError(e){
            if(opts.onError) opts.onError(e);
        };
        return request;
    };
/* Capitalize */
    cuppa.capitalize = function(string, firstWord) {
        if(firstWord) return string.charAt(0).toUpperCase() + string.slice(1);

        var pieces = string.split(" ");
        for ( var i = 0; i < pieces.length; i++ ) { var j = pieces[i].charAt(0).toUpperCase(); pieces[i] = j + pieces[i].substr(1); }
        return pieces.join(" ");
    }
/* Add EnterFrame listener */
    cuppa.enter_frame_list = [];
    cuppa.enterFrame = function(callback, fps, groupName){ cuppa.addEnterFrame(callback, fps, groupName); }
    cuppa.tick = function(callback, fps, groupName){ cuppa.addEnterFrame(callback, fps, groupName); }
    cuppa.addEnterFrame = function(callback, fps, groupName){
        if(fps == undefined) fps = 60;
        if(groupName == undefined) groupName = "default";
        var fps = (1000/fps)/1000;
        //++ Crate instance
            var instance = {}
                instance.callback = callback;
                instance.fps = fps;
                instance.groupName = groupName;
                instance["function"] = function(instance){
                    callback();
                    window.TweenMax.delayedCall(instance.fps, instance["function"], [instance] );
                };
               instance["function"](instance);
        //--
        cuppa.enter_frame_list.push(instance);
    };
    cuppa.removeEnterFrame = function(callback){
        var i;
        for(var i = 0; i < cuppa.enter_frame_list.length; i++){
            var instance = cuppa.enter_frame_list[i];
            if(instance.callback === callback){
                window.TweenMax.killTweensOf(instance["function"]);
                break;
            };
        };
        cuppa.enter_frame_list.splice(i, 1);
    };
    cuppa.removeEnterFrameGroup = function(groupName){
        if(groupName == undefined) groupName = "default";
        for(var i = 0; i < cuppa.enter_frame_list.length; i++){
            var instance = cuppa.enter_frame_list[i];
            if(instance.groupName == groupName){
                window.TweenMax.killTweensOf(instance["function"]);
            };
        };
    };
/* timer
    timer = new timer(opts);
    opts = {
        callback:function(item){}
    }
*/
    cuppa.timer = function(opts){
        this.hours = 0; this.minuts = 0; this.seconds = 0;
        this.running = false;
        this.interval = null;
        this.opts = opts || {};
        this.calculate = function(){
            this.seconds += 1;
            if(this.seconds == 60){
                this.seconds = 0;
                this.minuts +=1;
            }
            if (this.minuts == 60) {
                this.seconds = 0;
                this.minuts = 0;
                this.hours += 1;
            }
            if(this.opts.callback) this.opts.callback(this);
        }.bind(this);

        this.start = function(reset){
            this.destroy();
            if(reset) this.reset();
            this.interval = setInterval(this.calculate, 1000);
            this.running = true;
            if(this.opts.callback) this.opts.callback(this);
        }.bind(this);

        this.stop = function(reset){
            this.destroy();
            if(reset) this.reset();
            this.running = false;
        }.bind(this);

        this.reset = function(){
            this.hours = 0; this.minuts = 0; this.seconds = 0;
        }.bind(this);

        this.getSeconds = function(){
            return ((this.hours*60*60)+(this.minuts*60)+this.seconds);
        }.bind(this);

        this.destroy = function(){
            if(this.interval){ clearInterval(this.interval); this.interval = null; };
        }.bind(this);
    };

/* sizeFormat */
    cuppa.sizeFormat = function(bytes, si){
        if(si === undefined) si = true;
        var thresh = si ? 1000 : 1024;
        if(Math.abs(bytes) < thresh) {
            return bytes + ' B';
        }
        var units = si
            ? ['KB','MB','GB','TB','PB','EB','ZB','YB']
            : ['KiB','MiB','GiB','TiB','PiB','EiB','ZiB','YiB'];
        var u = -1;
        do {
            bytes /= thresh;
            ++u;
        } while(Math.abs(bytes) >= thresh && u < units.length - 1);
        return bytes.toFixed(1)+' '+units[u];
    }
/* cover
    opts = {
        invert:false,
        force: width, height
    }
*/
    cuppa.coverImage = function(elements, opts){
        this.opts = opts || {};
        if(elements == undefined) elements = "img.cover";
        this.elements = cuppa.element(elements, {query:true});
        cuppa.css(this.elements, {opacity:0, position:"absolute"});

        this.resize = function(){
            for(var i = 0; i < this.elements.length; i++){
                var element = this.elements[i];
                if(element.realWidth && element.realHeight){
                    var parent = cuppa.parent(element);
                    var parentDim = cuppa.dim(parent);
                    var nDim = cuppa.newDim(element.realWidth, element.realHeight, parentDim.width, parentDim.height, this.opts.invert, this.opts.force);
                    var xPos = (parentDim.width - nDim.width)*0.5;
                    var yPos = (parentDim.height - nDim.height)*0.5;
                    cuppa.css(element, {width:nDim.width+"px", height:nDim.height+"px", left:xPos+"px", top:yPos+"px"});
                }
            }
        }.bind(this); cuppa.on(window, "resize", this.resize);

        this.processImage = function(element){
            if(element.currentTarget) element = element.currentTarget;
            element.realWidth = element.width;
            element.realHeight = element.height;
            this.resize();
            window.TweenMax.to(element, 0.3, {alpha:1});
        }.bind(this); cuppa.on(this.elements, "load", this.processImage);

        cuppa.on(window, "load", function(){
            for(var j = 0; j < this.elements.length; j++){ this.processImage(this.elements[j]); };
        }.bind(this));
    };
/* copy
    element = html node, input, textarea
*/
    cuppa.copy = function(element, opts){
        opts = cuppa.mergeObjects([{success:null, error:null, alert:false, innerHTML:false}, opts]);
        if(opts.callback) opts.success = opts.callback;
        element = cuppa.element(element)[0];
        if(cuppa.elementType(element) != "textarea" && cuppa.elementType(element) != "input"){
            var textArea;
            if(opts.innerHTML) textArea = cuppa.newElement("<textarea style='position:absolute; top:0; left:0; width:0; height:0; opacity:0'>"+element.innerHTML+"</textarea>");
            else  textArea = cuppa.newElement("<textarea style='position:absolute; top:0; left:0; width:0; height:0; opacity:0'>"+element.outerHTML+"</textarea>");
                cuppa.append(textArea, document.body);
                textArea.select();
                try { document.execCommand('copy'); if(opts.success) opts.success(); }catch(err){ if(opts.error) opts.error(); }
                cuppa.remove(textArea);
        }else{
            var textArea = cuppa.newElement("<textarea style='position:absolute; top:0; left:0; width:0; height:0; opacity:0'>"+element.value+"</textarea>");
                cuppa.append(textArea, document.body);
                textArea.select();
                try { document.execCommand('copy'); if(opts.success) opts.success(); }catch(err){ if(opts.error) opts.error(); }
                cuppa.remove(textArea);
        };
        if(opts.alert) alert(opts.alert);
    }.bind(this);
/* anchorSmooth
        element = 'a' default;
*/
    cuppa.anchorSmooth = function(elements, opts){
        this.opts = cuppa.mergeObjects([{ duration:0.7, ease:window.Expo.easeInOut, gap:0, callback:null }, opts]);
        if(!elements) elements = "a";
        this.elements = cuppa.element(elements);
        this.onClick = function(e){
            var href = cuppa.attr(e.currentTarget, "href");
            if(!href) return;
            var anchor_str = href.split("#");
                anchor_str = anchor_str.pop();
            var anchor = null;
            if(!anchor) anchor = cuppa.element("a[name="+anchor_str+"]")[0];
            if(!anchor) return;

            e.preventDefault();
            var dim = cuppa.dim(anchor);
            cuppa.moveScroll(document.body, { x:0, y:dim.y + this.opts.gap, duration:this.opts.duration, ease:this.opts.ease });
            cuppa.setPath(href);
            if(this.opts.callback) this.opts.callback.callback(anchor_str);
        }.bind(this); cuppa.on(this.elements, "click", this.onClick);
    };
// includeHTML
    cuppa.includeHTML = function(elements, opts){
        //opts = cuppa.mergeObjects([{}, opts]);
        if(!elements) elements = "[cu-include]";
        this.elements = cuppa.element(elements);

        this.process = function(element){
            var url = cuppa.attribute(element, "cu-include");
            cuppa.ajax(url, {callback:function(result){
                var html = cuppa.newElement(result);
                if(cuppa.attribute(element, "cu-replace") != undefined){
                    cuppa.replaceElement(html, element);
                }else{
                    cuppa.append(html, element);
                }
            }});
        }.bind(this);
        this.elements.forEach(this.process);
    };
// selectText
    cuppa.selectText = function(element, start, end){
        start = start || 0;
        end = end || element.value.length;
        element.setSelectionRange(start, end);
    };
/* tabs
    <div id="tab1" class="tab" tab-content="#tabContent1" >Tab1</div>
    <div id="tab2" class="tab" tab-content="#tabContent2" >Tab2</div>

    new cuppa.tabs(this.html.querySelectorAll(".tab"), {height:true});

* */
    cuppa.tabs = function(elements, opts){
        this.elements = cuppa.element(elements);
        this.elementContents = [];
        this.opts = cuppa.mergeObjects([{display:"block", height:false}, opts]);

        this.hide = function(elements){
            if(!elements) elements = this.elementContents;

            if(this.opts.height) cuppa.css(elements, {height:"0px"});
            else cuppa.hide(elements, {duration:0});
        }.bind(this);

        this.show = function(e){
            if(e.currentTarget) e = e.currentTarget;
            this.hide();
            cuppa.removeClass(this.elements, "selected");
            cuppa.addClass(e, "selected");
            if(this.opts.height) cuppa.css(e.tabContent, {height:"auto"});
            else cuppa.show(e.tabContent, {duration:0});
        }.bind(this);

        // process
            this.elements.forEach(function(item, index){
                // get tab content
                    var tabContent = cuppa.attr(item, "tab-content");
                        tabContent = cuppa.element(tabContent)[0];
                    this.elementContents.push(tabContent);
                    item.tabContent = tabContent;
                // hide tab
                    this.hide();
            }.bind(this));
        // init
            this.show(this.elements[0]);
            cuppa.on(this.elements, "click", this.show);

    };
/* language */
    cuppa.lang = function(){ return (document.documentElement.lang || "").toLowerCase(); };
    cuppa.language = function(){ return cuppa.lang(); }
    cuppa.currentLanguage = function(){ return cuppa.lang(); }
/* accordeon
    example: new cuppa.accordion('.content', '.btn')
*/
    cuppa.accordion = function(element, btn , opts){
        this.element = cuppa.element(element)[0];
        this.btn = cuppa.element(btn)[0];
        this.opts = cuppa.mergeObjects([{init:"collapsed", callback:null}, opts]);
        this.childContent = cuppa.childrens(this.element)[0];
        if(this.opts.init == "collapsed") cuppa.css(this.element, {height:"0px"});
        this.element.accordion = this.btn.accordion = this;
        this.open = function(value){
            var dim = cuppa.dim(this.element);
            if(value.currentTarget || value == undefined){
                if(!dim.height) value = true;
                else value = false;
            };
            var dim2 = cuppa.dim(this.childContent );
            window.TweenMax.killTweensOf([this.element, this.btn]);
            if(value){
                var tl = new window.TimelineMax();
                    tl.add(function(){ cuppa.addClass([this.element, this.btn], "open") }.bind(this));
                    tl.to(this.element, 0.4, {height:dim2.height, ease:window.Cubic.easeInOut});
                    tl.set(this.element, {height:"auto"});
            }else{
                var tl = new window.TimelineMax();
                    tl.set(this.element, {height:dim2.height});
                    tl.to(this.element, 0.4, {height:0, ease:window.Cubic.easeInOut});
                    tl.add(function(){ cuppa.removeClass([this.element, this.btn], "open") }.bind(this));
            };
            if(this.opts.callback) this.opts.callback(this.btn);
        }.bind(this);
        this.isOpen = function(){
            return cuppa.hasClass(this.btn, "open");
        }.bind(this);
        cuppa.on(this.btn, "click", this.open);
    };
/* browser
     cuppa.browser() // return edge
*/
    cuppa.browser = function() {
        var userAgentString = navigator.userAgent;
        var browsers = [
            [ 'edge', /Edge\/([0-9\._]+)/ ],
            [ 'yandexbrowser', /YaBrowser\/([0-9\._]+)/ ],
            [ 'vivaldi', /Vivaldi\/([0-9\.]+)/ ],
            [ 'kakaotalk', /KAKAOTALK\s([0-9\.]+)/ ],
            [ 'chrome', /(?!Chrom.*OPR)Chrom(?:e|ium)\/([0-9\.]+)(:?\s|$)/ ],
            [ 'crios', /CriOS\/([0-9\.]+)(:?\s|$)/ ],
            [ 'firefox', /Firefox\/([0-9\.]+)(?:\s|$)/ ],
            [ 'fxios', /FxiOS\/([0-9\.]+)/ ],
            [ 'opera', /Opera\/([0-9\.]+)(?:\s|$)/ ],
            [ 'opera', /OPR\/([0-9\.]+)(:?\s|$)$/ ],
            [ 'ie', /Trident\/7\.0.*rv\:([0-9\.]+).*\).*Gecko$/ ],
            [ 'ie', /MSIE\s([0-9\.]+);.*Trident\/[4-7].0/ ],
            [ 'ie', /MSIE\s(7\.0)/ ],
            [ 'bb10', /BB10;\sTouch.*Version\/([0-9\.]+)/ ],
            [ 'android', /Android\s([0-9\.]+)/ ],
            [ 'ios', /Version\/([0-9\._]+).*Mobile.*Safari.*/ ],
            [ 'safari', /Version\/([0-9\._]+).*Safari/ ]
        ];
        return browsers.map(function (rule) {
            if (rule[1].test(userAgentString)) {
                var match = rule[1].exec(userAgentString);
                var version = match && match[1].split(/[._]/).slice(0,3);

                if (version && version.length < 3) {
                    Array.prototype.push.apply(version, (version.length == 1) ? [0, 0] : [0]);
                }

                return {
                    name: rule[0],
                    version: version.join('.')
                };
            }
        }).filter(Boolean).shift();
    };
// cuppa OS
    cuppa.OS = function(){
        return window.jscd;
    };
// Google conversion Adwords
    cuppa.googleConvertion = function(google_conversion_id, google_conversion_label) {
        var image = new Image(1, 1);
            image.src = "//www.googleadservices.com/pagead/conversion/" + google_conversion_id + "/?label=" + google_conversion_label + "&script=0";
        log("cuppa.googleConvertion | id:" + google_conversion_id + " | label: " + google_conversion_label);
    };
    cuppa.gc = function(google_conversion_id, google_conversion_label){ cuppa.googleConvertion(google_conversion_id, google_conversion_label); };
// Shake
    cuppa.shake = function(element){
        window.TweenMax.to(element, 0.1, {left:10});
        window.TweenMax.to(element, 0.1, {delay:0.1, left:-10});
        window.TweenMax.to(element, 0.1, {delay:0.2, left:0});
    };
// isJson
    cuppa.isJson = function(item) {
        try { item = JSON.parse(item); } catch (e) { item = null; }
        if(item !== null) return true;
        return false;
    };